@@ -23,80 +23,44 @@
 
 namespace folly {
 
-template <class Fn>
-struct exception_wrapper::arg_type_
-    : public arg_type_<decltype(&Fn::operator())> {};
-template <class Ret, class Class, class Arg>
-struct exception_wrapper::arg_type_<Ret (Class::*)(Arg)> {
-  using type = Arg;
-};
-template <class Ret, class Class, class Arg>
-struct exception_wrapper::arg_type_<Ret (Class::*)(Arg) const> {
-  using type = Arg;
-};
-template <class Ret, class Arg>
-struct exception_wrapper::arg_type_<Ret(Arg)> {
-  using type = Arg;
-};
-template <class Ret, class Arg>
-struct exception_wrapper::arg_type_<Ret (*)(Arg)> {
-  using type = Arg;
-};
-template <class Ret, class Class>
-struct exception_wrapper::arg_type_<Ret (Class::*)(...)> {
-  using type = void;
-};
-template <class Ret, class Class>
-struct exception_wrapper::arg_type_<Ret (Class::*)(...) const> {
-  using type = void;
-};
-template <class Ret>
-struct exception_wrapper::arg_type_<Ret(...)> {
-  using type = void;
-};
-template <class Ret>
-struct exception_wrapper::arg_type_<Ret (*)(...)> {
-  using type = void;
-};
-
-#ifdef FOLLY_HAVE_NOEXCEPT_FUNCTION_TYPE
-template <class Ret, class Class, class Arg>
-struct exception_wrapper::arg_type_<Ret (Class::*)(Arg) noexcept> {
-  using type = Arg;
-};
-template <class Ret, class Class, class Arg>
-struct exception_wrapper::arg_type_<Ret (Class::*)(Arg) const noexcept> {
-  using type = Arg;
-};
-template <class Ret, class Arg>
-struct exception_wrapper::arg_type_<Ret(Arg) noexcept> {
-  using type = Arg;
-};
-template <class Ret, class Arg>
-struct exception_wrapper::arg_type_<Ret (*)(Arg) noexcept> {
-  using type = Arg;
-};
-template <class Ret, class Class>
-struct exception_wrapper::arg_type_<Ret (Class::*)(...) noexcept> {
-  using type = void;
-};
-template <class Ret, class Class>
-struct exception_wrapper::arg_type_<Ret (Class::*)(...) const noexcept> {
-  using type = void;
-};
-template <class Ret>
-struct exception_wrapper::arg_type_<Ret(...) noexcept> {
-  using type = void;
-};
-template <class Ret>
-struct exception_wrapper::arg_type_<Ret (*)(...) noexcept> {
-  using type = void;
-};
-#endif
-
 struct exception_wrapper::with_exception_from_fn_ {
+  struct impl_var_ {
+    template <typename>
+    using apply = void;
+  };
+  struct impl_arg_ {
+    template <typename F>
+    using apply = typename function_traits<F>::template argument<0>;
+  };
+  struct impl_bye_;
+  template <
+      typename Sig,
+      typename Traits = function_traits<Sig>,
+      std::size_t NArgs = Traits::template arguments<type_pack_size_t>::value>
+  using impl_ = conditional_t<
+      Traits::is_variadic,
+      impl_var_,
+      conditional_t<NArgs == 1, impl_arg_, impl_bye_>>;
+
+  template <typename T>
+  using member_ = typename member_pointer_traits<T>::member_type;
+
+  template <typename Void, typename>
+  struct arg_type_;
+  template <class Sig>
+  struct arg_type_<std::enable_if_t<std::is_function<Sig>::value>, Sig> {
+    using type = typename impl_<Sig>::template apply<Sig>;
+  };
+  template <class Ptr>
+  struct arg_type_<std::enable_if_t<std::is_pointer<Ptr>::value>, Ptr>
+      : arg_type_<void, std::remove_pointer_t<Ptr>> {};
+  template <class Obj>
+  struct arg_type_<void_t<decltype(&Obj::operator())>, Obj>
+      : arg_type_<void, member_<decltype(&Obj::operator())>> {};
+
+  // void if Fn is a variadic callable; otherwise the first arg type
   template <typename, typename Fn>
-  using apply = arg_type<Fn>;
+  using apply = _t<arg_type_<void, Fn>>;
 };
 
 struct exception_wrapper::with_exception_from_ex_ {
