@@ -41,6 +41,7 @@
 #include <folly/Traits.h>
 #include <folly/lang/CheckedMath.h>
 #include <folly/lang/Exception.h>
+#include <folly/lang/Hint.h>
 #include <folly/memory/Malloc.h>
 
 //=============================================================================
@@ -162,7 +163,7 @@ class fbvector {
     }
 
     void init(size_type n) {
-      if (UNLIKELY(n == 0)) {
+      if (FOLLY_UNLIKELY(n == 0)) {
         b_ = e_ = z_ = nullptr;
       } else {
         size_type sz = folly::goodMallocSize(n * sizeof(T)) / sizeof(T);
@@ -424,7 +425,7 @@ class fbvector {
   static void S_uninitialized_fill_n_a(
       Allocator& a, T* dest, size_type sz, Args&&... args) {
     auto b = dest;
-    T* e;
+    T* e = nullptr;
     if (!folly::checked_add(&e, dest, sz)) {
       throw_exception<std::length_error>("FBVector exceeded max size.");
     }
@@ -439,8 +440,8 @@ class fbvector {
   // optimized
   static void S_uninitialized_fill_n(T* dest, size_type n) {
     if (folly::IsZeroInitializable<T>::value) {
-      if (LIKELY(n != 0)) {
-        T* sz;
+      if (FOLLY_LIKELY(n != 0)) {
+        T* sz = nullptr;
         if (!folly::checked_add(&sz, dest, n)) {
           throw_exception<std::length_error>("FBVector exceeded max size.");
         }
@@ -448,7 +449,7 @@ class fbvector {
       }
     } else {
       auto b = dest;
-      T* e;
+      T* e = nullptr;
       if (!folly::checked_add(&e, dest, n)) {
         throw_exception<std::length_error>("FBVector exceeded max size.");
       }
@@ -467,7 +468,7 @@ class fbvector {
 
   static void S_uninitialized_fill_n(T* dest, size_type n, const T& value) {
     auto b = dest;
-    T* e;
+    T* e = nullptr;
     if (!folly::checked_add(&e, dest, n)) {
       throw_exception<std::length_error>("FBVector exceeded max size.");
     }
@@ -750,7 +751,7 @@ class fbvector {
   ~fbvector() = default; // the cleanup occurs in impl_
 
   fbvector& operator=(const fbvector& other) {
-    if (UNLIKELY(this == &other)) {
+    if (FOLLY_UNLIKELY(this == &other)) {
       return *this;
     }
 
@@ -768,7 +769,7 @@ class fbvector {
   }
 
   fbvector& operator=(fbvector&& other) {
-    if (UNLIKELY(this == &other)) {
+    if (FOLLY_UNLIKELY(this == &other)) {
       return *this;
     }
     moveFrom(std::move(other), moveIsSwap());
@@ -889,7 +890,7 @@ class fbvector {
     return dataIsInternal(t);
   }
   bool dataIsInternal(const T& t) {
-    return UNLIKELY(
+    return FOLLY_UNLIKELY(
         impl_.b_ <= std::addressof(t) && std::addressof(t) < impl_.e_);
   }
 
@@ -1055,7 +1056,7 @@ class fbvector {
     return impl_.b_[n];
   }
   const_reference at(size_type n) const {
-    if (UNLIKELY(n >= size())) {
+    if (FOLLY_UNLIKELY(n >= size())) {
       throw_exception<std::out_of_range>(
           "fbvector: index is greater than size.");
     }
@@ -1349,6 +1350,7 @@ class fbvector {
       impl_.e_ += n;
     } else {
       if (folly::IsRelocatable<T>::value && usingStdAllocator) {
+        compiler_may_unsafely_assume(position != nullptr);
         std::memmove((void*)(position + n), (void*)position, tail * sizeof(T));
         impl_.e_ += n;
       } else {
