@@ -18,8 +18,12 @@
 
 #if !defined(__AVX2__) || defined(__aarch64__)
 
+namespace folly {
+
 extern "C" void* __folly_memset(void* dest, int ch, std::size_t count) {
   return std::memset(dest, ch, count);
 }
 
+} // namespace folly
+
 #endif
