@@ -41,7 +41,7 @@ namespace folly {
 namespace detail {
 
 // MSVC does not implement noexcept deduction https://godbolt.org/z/Mxdjao1q6
-#if defined FOLLY_HAVE_NOEXCEPT_FUNCTION_TYPE && !defined _MSC_VER
+#if !defined(_MSC_VER)
 #define FOLLY_DETAIL_NOEXCEPT_SPECIFICATION noexcept(Noexcept)
 #define FOLLY_DETAIL_NOEXCEPT_DECLARATION bool Noexcept,
 #else
@@ -306,15 +306,12 @@ decltype(auto) variant_match(Variant&& variant, Cases&&... cases) {
 template <typename R, typename Variant, typename... Cases>
 R variant_match(Variant&& variant, Cases&&... cases) {
   auto f = [&](auto&& v) -> R {
-#if __cpp_if_constexpr >= 201606L
     if constexpr (std::is_void<R>::value) {
-      overload(std::forward<Cases>(cases)...)(v);
+      overload(std::forward<Cases>(cases)...)(std::forward<decltype(v)>(v));
     } else {
-      return overload(std::forward<Cases>(cases)...)(v);
+      return overload(std::forward<Cases>(cases)...)(
+          std::forward<decltype(v)>(v));
     }
-#else
-    return static_cast<R>(overload(std::forward<Cases>(cases)...)(v));
-#endif
   };
   using invoker = std::conditional_t<
       folly::Conjunction<
