@@ -32,7 +32,7 @@
 #include <version>
 #endif
 
-static_assert(FOLLY_CPLUSPLUS >= 201402L, "__cplusplus >= 201402L");
+static_assert(FOLLY_CPLUSPLUS >= 201703L, "__cplusplus >= 201703L");
 
 #if defined(__GNUC__) && !defined(__clang__)
 static_assert(__GNUC__ >= 7, "__GNUC__ >= 7");
@@ -86,6 +86,9 @@ constexpr bool kHasUnalignedAccess = false;
 #define FOLLY_NODISCARD
 #endif
 
+// older clang-format gets confused by [[deprecated(...)]] on class decls
+#define FOLLY_DEPRECATED(...) [[deprecated(__VA_ARGS__)]]
+
 // target
 #ifdef _MSC_VER
 #define FOLLY_TARGET_ATTRIBUTE(target)
@@ -518,16 +521,6 @@ constexpr auto kCpplibVer = 0;
 #define FOLLY_STORAGE_CONSTEXPR constexpr
 #endif
 
-//  FOLLY_CXX17_CONSTEXPR
-//
-//  C++17 permits more cases to be marked constexpr, including lambda bodies and
-//  the `if` keyword.
-#if FOLLY_CPLUSPLUS >= 201703L
-#define FOLLY_CXX17_CONSTEXPR constexpr
-#else
-#define FOLLY_CXX17_CONSTEXPR
-#endif
-
 //  FOLLY_CXX20_CONSTEXPR
 //
 //  C++20 permits more cases to be marked constexpr, including constructors that
@@ -584,17 +577,9 @@ constexpr auto kCpplibVer = 0;
 #endif // FOLLY_CPLUSPLUS >= 201703L
 #endif // FOLLY_CFG_NO_COROUTINES
 
-// MSVC 2017.5 && C++17
-#if __cpp_noexcept_function_type >= 201510 || \
-    (_MSC_FULL_VER >= 191225816 && _MSVC_LANG > 201402)
-#define FOLLY_HAVE_NOEXCEPT_FUNCTION_TYPE 1
-#endif
-
 #if __cpp_inline_variables >= 201606L || FOLLY_CPLUSPLUS >= 201703L
-#define FOLLY_HAS_INLINE_VARIABLES 1
 #define FOLLY_INLINE_VARIABLE inline
 #else
-#define FOLLY_HAS_INLINE_VARIABLES 0
 #define FOLLY_INLINE_VARIABLE
 #endif
 
@@ -612,16 +597,3 @@ constexpr auto kCpplibVer = 0;
 #else
 #define FOLLY_CONSTEVAL constexpr
 #endif
-
-// C++17 deduction guides
-#if defined(__cpp_deduction_guides) && __cpp_deduction_guides >= 201703L
-#define FOLLY_HAS_DEDUCTION_GUIDES 1
-#else
-#define FOLLY_HAS_DEDUCTION_GUIDES 0
-#endif
-// C++20 ranges
-#if defined(__cpp_lib_ranges)
-#define FOLLY_HAS_RANGES 1
-#else
-#define FOLLY_HAS_RANGES 0
-#endif
