@@ -14,6 +14,10 @@
  * limitations under the License.
  */
 
+//
+// Docs: https://fburl.com/fbcref_synchronized
+//
+
 /**
  * This module implements a Synchronized abstraction useful in
  * mutex-based concurrency.
@@ -1004,6 +1008,28 @@ struct Synchronized : public SynchronizedBase<
   mutable Mutex mutex_;
 };
 
+/**
+ * Deprecated subclass of Synchronized that provides implicit locking
+ * via operator->. This is intended to ease migration while preventing
+ * accidental use of operator-> in new code.
+ */
+template <class T, class Mutex = SharedMutex>
+struct [[deprecated(
+    "use Synchronized and explicit lock(), wlock(), or rlock() instead")]] ImplicitSynchronized
+    : Synchronized<T, Mutex> {
+ private:
+  using Base = Synchronized<T, Mutex>;
+
+ public:
+  using LockedPtr = typename Base::LockedPtr;
+  using ConstLockedPtr = typename Base::ConstLockedPtr;
+  using DataType = typename Base::DataType;
+  using MutexType = typename Base::MutexType;
+
+  using Base::Base;
+  using Base::operator=;
+};
+
 template <class SynchronizedType, class LockPolicy>
 class ScopedUnlocker;
 
@@ -1642,9 +1668,7 @@ void lock(LockableOne& one, LockableTwo& two, Lockables&... lockables) {
         lockable,
         [](auto& l) { return std::unique_lock<Lockable>{l}; },
         [](auto& l) {
-          auto lock = std::unique_lock<Lockable>{l, std::defer_lock};
-          lock.try_lock();
-          return lock;
+          return std::unique_lock<Lockable>{l, std::try_to_lock};
         });
   };
   auto locks = lock(locker(one), locker(two), locker(lockables)...);
