@@ -88,6 +88,55 @@ FOLLY_INLINE_VARIABLE constexpr bool always_false = false;
 
 namespace detail {
 
+template <typename Void, typename T>
+struct require_sizeof_ {
+  static_assert(always_false<T>, "application of sizeof fails substitution");
+};
+template <typename T>
+struct require_sizeof_<decltype(void(sizeof(T))), T> {
+  template <typename V>
+  using apply_t = V;
+
+  static constexpr std::size_t size = sizeof(T);
+};
+
+} // namespace detail
+
+//  require_sizeof
+//
+//  Equivalent to sizeof, but with a static_assert enforcing that application of
+//  sizeof would not fail substitution.
+template <typename T>
+constexpr std::size_t require_sizeof = detail::require_sizeof_<void, T>::size;
+
+//  is_unbounded_array_v
+//  is_unbounded_array
+//
+//  A trait variable and type to check if a given type is an unbounded array.
+//
+//  mimic: std::is_unbounded_array_d, std::is_unbounded_array (C++20)
+template <typename T>
+FOLLY_INLINE_VARIABLE constexpr bool is_unbounded_array_v = false;
+template <typename T>
+FOLLY_INLINE_VARIABLE constexpr bool is_unbounded_array_v<T[]> = true;
+template <typename T>
+struct is_unbounded_array : bool_constant<is_unbounded_array_v<T>> {};
+
+//  is_bounded_array_v
+//  is_bounded_array
+//
+//  A trait variable and type to check if a given type is a bounded array.
+//
+//  mimic: std::is_bounded_array_d, std::is_bounded_array (C++20)
+template <typename T>
+FOLLY_INLINE_VARIABLE constexpr bool is_bounded_array_v = false;
+template <typename T, std::size_t S>
+FOLLY_INLINE_VARIABLE constexpr bool is_bounded_array_v<T[S]> = true;
+template <typename T>
+struct is_bounded_array : bool_constant<is_bounded_array_v<T>> {};
+
+namespace detail {
+
 //  is_instantiation_of_v
 //  is_instantiation_of
 //
@@ -115,6 +164,19 @@ struct is_similar_instantiation
 
 } // namespace detail
 
+//  member_pointer_traits
+//
+//  For a member-pointer, reveals its constituent member-type and object-type.
+//
+//  Works for both member-object-pointer and member-function-pointer.
+template <typename>
+struct member_pointer_traits;
+template <typename M, typename O>
+struct member_pointer_traits<M O::*> {
+  using member_type = M;
+  using object_type = O;
+};
+
 namespace detail {
 
 struct is_constexpr_default_constructible_ {
@@ -135,7 +197,7 @@ struct is_constexpr_default_constructible_ {
   static std::false_type sfinae(void*);
   template <typename T>
   static constexpr bool apply =
-      decltype(sfinae(static_cast<T*>(nullptr)))::value;
+      !require_sizeof<T> || decltype(sfinae(static_cast<T*>(nullptr)))::value;
 };
 
 } // namespace detail
@@ -342,6 +404,8 @@ struct detected_<void_t<T<A...>>, D, T, A...> {
 //  alias value_t as std::false_type and has member type alias type as D.
 //
 //  mimic: std::experimental::detected_or, Library Fundamentals TS v2
+//
+//  Note: not resilient against incomplete types; may violate ODR.
 template <typename D, template <typename...> class T, typename... A>
 using detected_or = detail::detected_<void, D, T, A...>;
 
@@ -353,6 +417,8 @@ using detected_or = detail::detected_<void, D, T, A...>;
 //  Equivalent to detected_or<D, T, A...>::type.
 //
 //  mimic: std::experimental::detected_or_t, Library Fundamentals TS v2
+//
+//  Note: not resilient against incomplete types; may violate ODR.
 template <typename D, template <typename...> class T, typename... A>
 using detected_or_t = typename detected_or<D, T, A...>::type;
 
@@ -364,6 +430,8 @@ using detected_or_t = typename detected_or<D, T, A...>::type;
 //  Equivalent to detected_or_t<nonesuch, T, A...>.
 //
 //  mimic: std::experimental::detected_t, Library Fundamentals TS v2
+//
+//  Note: not resilient against incomplete types; may violate ODR.
 template <template <typename...> class T, typename... A>
 using detected_t = detected_or_t<nonesuch, T, A...>;
 
@@ -381,6 +449,8 @@ using detected_t = detected_or_t<nonesuch, T, A...>;
 //  mimic: std::experimental::is_detected, std::experimental::is_detected_v,
 //    Library Fundamentals TS v2
 //
+//  Note: not resilient against incomplete types; may violate ODR.
+//
 //  Note: the trait type is_detected differs here by being deferred.
 template <template <typename...> class T, typename... A>
 FOLLY_INLINE_VARIABLE constexpr bool is_detected_v =
@@ -405,6 +475,37 @@ template <class T>
 FOLLY_INLINE_VARIABLE constexpr bool is_trivially_copyable_v =
     is_trivially_copyable<T>::value;
 
+//  ----
+
+namespace fallback {
+template <typename From, typename To>
+FOLLY_INLINE_VARIABLE constexpr bool is_nothrow_convertible_v =
+    (std::is_void<From>::value && std::is_void<To>::value) ||
+    ( //
+        std::is_convertible<From, To>::value &&
+        std::is_nothrow_constructible<To, From>::value);
+template <typename From, typename To>
+struct is_nothrow_convertible
+    : bool_constant<is_nothrow_convertible_v<From, To>> {};
+} // namespace fallback
+
+//  is_nothrow_convertible
+//  is_nothrow_convertible_v
+//
+//  Import or backport:
+//  * std::is_nothrow_convertible
+//  * std::is_nothrow_convertible_v
+//
+//  mimic: is_nothrow_convertible, C++20
+#if defined(__cpp_lib_is_nothrow_convertible) && \
+    __cpp_lib_is_nothrow_convertible >= 201806L
+using std::is_nothrow_convertible;
+using std::is_nothrow_convertible_v;
+#else
+using fallback::is_nothrow_convertible;
+using fallback::is_nothrow_convertible_v;
+#endif
+
 /**
  * IsRelocatable<T>::value describes the ability of moving around
  * memory a value of type T by using memcpy (as opposed to the
@@ -443,8 +544,8 @@ FOLLY_INLINE_VARIABLE constexpr bool is_trivially_copyable_v =
  * It may be unset in a base class by overriding the typedef to false_type.
  */
 /*
- * IsZeroInitializable describes the property that default construction is the
- * same as memset(dst, 0, sizeof(T)).
+ * IsZeroInitializable describes the property that value-initialization
+ * is the same as memset(dst, 0, sizeof(T)).
  */
 
 namespace traits_detail {
@@ -534,7 +635,8 @@ struct IsNothrowSwappable
 template <class T>
 struct IsRelocatable
     : std::conditional<
-          is_detected_v<traits_detail::detect_IsRelocatable, T>,
+          !require_sizeof<T> ||
+              is_detected_v<traits_detail::detect_IsRelocatable, T>,
           traits_detail::has_true_IsRelocatable<T>,
           // TODO add this line (and some tests for it) when we
           // upgrade to gcc 4.7
@@ -544,7 +646,8 @@ struct IsRelocatable
 template <class T>
 struct IsZeroInitializable
     : std::conditional<
-          is_detected_v<traits_detail::detect_IsZeroInitializable, T>,
+          !require_sizeof<T> ||
+              is_detected_v<traits_detail::detect_IsZeroInitializable, T>,
           traits_detail::has_true_IsZeroInitializable<T>,
           bool_constant< //
               !std::is_class<T>::value && //
@@ -632,7 +735,7 @@ struct is_transparent : bool_constant<is_transparent_v<T>> {};
 namespace detail {
 
 template <typename T, typename = void>
-FOLLY_INLINE_VARIABLE constexpr bool is_allocator_ = false;
+FOLLY_INLINE_VARIABLE constexpr bool is_allocator_ = !require_sizeof<T>;
 template <typename T>
 FOLLY_INLINE_VARIABLE constexpr bool is_allocator_<
     T,
@@ -1012,4 +1115,10 @@ using type_pack_element_t = traits_detail::type_pack_element_fallback<I, Ts...>;
 
 #endif
 
+template <typename... Ts>
+FOLLY_INLINE_VARIABLE constexpr std::size_t type_pack_size_v = sizeof...(Ts);
+
+template <typename... Ts>
+using type_pack_size_t = index_constant<sizeof...(Ts)>;
+
 } // namespace folly
