@@ -462,12 +462,13 @@ class SimpleAllocator {
 
     std::lock_guard<std::mutex> g(allocator->m_);
     *static_cast<void**>(ptr) = allocator->freelist_;
-    if FOLLY_CXX17_CONSTEXPR (kIsSanitizeAddress) {
+    if constexpr (kIsSanitizeAddress) {
       // If running under ASAN, scrub the memory on deallocation, so we don't
       // leave pointers that could hide leaks at shutdown, since the backing
       // slabs may not be deallocated if the instance is a leaky singleton.
       auto* base = static_cast<char*>(ptr);
-      std::fill(base + sizeof(void*), base + allocator->sz_, 0);
+      std::fill(
+          base + sizeof(void*), base + allocator->sz_, static_cast<char>(0));
     }
     allocator->freelist_ = ptr;
   }
