@@ -41,15 +41,17 @@
 
 #include <folly/container/F14Map-fwd.h>
 #include <folly/container/Iterator.h>
+#include <folly/container/detail/F14MapFallback.h>
 #include <folly/container/detail/F14Policy.h>
 #include <folly/container/detail/F14Table.h>
 #include <folly/container/detail/Util.h>
 
+namespace folly {
+
 #if FOLLY_F14_VECTOR_INTRINSICS_AVAILABLE
 
 //////// Common case for supported platforms
 
-namespace folly {
 namespace f14 {
 namespace detail {
 
@@ -261,7 +263,7 @@ class F14BasicMap {
    * Remove all elements.
    * @methodset Modifiers
    *
-   * Does not free heap-allocated memory; capacity is unchanged.
+   * Frees heap-allocated memory; bucket_count is returned to 0.
    */
   void clear() noexcept { table_.clear(); }
 
@@ -1114,8 +1116,8 @@ class F14ValueMap
     this->table_.visitContiguousItemRanges(visitor);
   }
 };
+#endif // FOLLY_F14_VECTOR_INTRINSICS_AVAILABLE
 
-#if FOLLY_HAS_DEDUCTION_GUIDES
 template <
     typename InputIt,
     typename Hasher = f14::DefaultHasher<iterator_key_type_t<InputIt>>,
@@ -1199,8 +1201,8 @@ template <
 F14ValueMap(
     std::initializer_list<std::pair<Key, Mapped>>, std::size_t, Hasher, Alloc)
     -> F14ValueMap<Key, Mapped, Hasher, f14::DefaultKeyEqual<Key>, Alloc>;
-#endif
 
+#if FOLLY_F14_VECTOR_INTRINSICS_AVAILABLE
 template <
     typename Key,
     typename Mapped,
@@ -1263,8 +1265,8 @@ class F14NodeMap
 
   // TODO extract and node_handle insert
 };
+#endif // FOLLY_F14_VECTOR_INTRINSICS_AVAILABLE
 
-#if FOLLY_HAS_DEDUCTION_GUIDES
 template <
     typename InputIt,
     typename Hasher = f14::DefaultHasher<iterator_key_type_t<InputIt>>,
@@ -1347,8 +1349,8 @@ template <
 F14NodeMap(
     std::initializer_list<std::pair<Key, Mapped>>, std::size_t, Hasher, Alloc)
     -> F14NodeMap<Key, Mapped, Hasher, f14::DefaultKeyEqual<Key>, Alloc>;
-#endif
 
+#if FOLLY_F14_VECTOR_INTRINSICS_AVAILABLE
 namespace f14 {
 namespace detail {
 template <
@@ -1670,8 +1672,8 @@ class F14VectorMap : public f14::detail::F14VectorMapImpl<
     return {c.rbegin(), c.rend()};
   }
 };
+#endif // FOLLY_F14_VECTOR_INTRINSICS_AVAILABLE
 
-#if FOLLY_HAS_DEDUCTION_GUIDES
 template <
     typename InputIt,
     typename Hasher = f14::DefaultHasher<iterator_key_type_t<InputIt>>,
@@ -1754,8 +1756,8 @@ template <
 F14VectorMap(
     std::initializer_list<std::pair<Key, Mapped>>, std::size_t, Hasher, Alloc)
     -> F14VectorMap<Key, Mapped, Hasher, f14::DefaultKeyEqual<Key>, Alloc>;
-#endif
 
+#if FOLLY_F14_VECTOR_INTRINSICS_AVAILABLE
 /**
  * F14FastMap is, under the hood, either an F14ValueMap or an F14VectorMap.
  * F14FastMap chooses which of these two representations to use based on the
@@ -1805,8 +1807,8 @@ class F14FastMap : public std::conditional_t<
     this->table_.swap(rhs.table_);
   }
 };
+#endif // if FOLLY_F14_VECTOR_INTRINSICS_AVAILABLE
 
-#if FOLLY_HAS_DEDUCTION_GUIDES
 template <
     typename InputIt,
     typename Hasher = f14::DefaultHasher<iterator_key_type_t<InputIt>>,
@@ -1889,14 +1891,9 @@ template <
 F14FastMap(
     std::initializer_list<std::pair<Key, Mapped>>, std::size_t, Hasher, Alloc)
     -> F14FastMap<Key, Mapped, Hasher, f14::DefaultKeyEqual<Key>, Alloc>;
-#endif
+
 } // namespace folly
 
-#endif // if FOLLY_F14_VECTOR_INTRINSICS_AVAILABLE
-
-//////// Compatibility for unsupported platforms (not x86_64 and not aarch64)
-#include <folly/container/detail/F14MapFallback.h>
-
 namespace folly {
 namespace f14 {
 namespace detail {
