@@ -23,6 +23,7 @@
 #include <type_traits>
 #include <utility>
 
+#include <folly/Traits.h>
 #include <folly/Utility.h>
 #include <folly/container/Access.h>
 #include <folly/lang/RValueReferenceWrapper.h>
@@ -34,7 +35,8 @@ namespace folly {
 //  Whether std::distance over a pair of iterators is reasonably known to give
 //  the distance without advancing the iterators or copies of them.
 template <typename Iter, typename SentinelIter>
-FOLLY_INLINE_VARIABLE constexpr bool iterator_has_known_distance_v = false;
+FOLLY_INLINE_VARIABLE constexpr bool iterator_has_known_distance_v =
+    !require_sizeof<Iter> || !require_sizeof<SentinelIter>;
 template <typename Iter>
 FOLLY_INLINE_VARIABLE constexpr bool iterator_has_known_distance_v<Iter, Iter> =
     std::is_base_of<
@@ -72,6 +74,58 @@ FOLLY_INLINE_VARIABLE constexpr bool range_has_known_distance_v =
         invoke_result_t<access::begin_fn, Range>,
         invoke_result_t<access::end_fn, Range>>;
 
+//  iterator_category_t
+//
+//  Extracts iterator_category from an iterator.
+template <typename Iter>
+using iterator_category_t =
+    typename std::iterator_traits<Iter>::iterator_category;
+
+namespace detail {
+
+template <typename Iter, typename Category, typename = void>
+FOLLY_INLINE_VARIABLE constexpr bool iterator_category_matches_v_ =
+    !require_sizeof<Iter> || !require_sizeof<Category>;
+template <typename Iter, typename Category>
+FOLLY_INLINE_VARIABLE constexpr bool iterator_category_matches_v_<
+    Iter,
+    Category,
+    void_t<iterator_category_t<Iter>>> =
+    std::is_convertible<iterator_category_t<Iter>, Category>::value;
+
+} // namespace detail
+
+//  iterator_category_matches_v
+//
+//  Whether an iterator's category matches Category (std::input_iterator_tag,
+//  std::output_iterator_tag, etc). Defined for non-iterator types as well.
+//
+//  Useful for containers deduction guides implementation.
+template <typename Iter, typename Category>
+FOLLY_INLINE_VARIABLE constexpr bool iterator_category_matches_v =
+    detail::iterator_category_matches_v_<Iter, Category>;
+
+//  iterator_value_type_t
+//
+//  Extracts a value type from an iterator.
+template <typename Iter>
+using iterator_value_type_t = typename std::iterator_traits<Iter>::value_type;
+
+//  iterator_key_type_t
+//
+//  Extracts a key type from an iterator, leverages the knowledge that
+//  key/value containers usually use std::pair<const K, V> as a value_type.
+template <typename Iter>
+using iterator_key_type_t =
+    remove_cvref_t<typename iterator_value_type_t<Iter>::first_type>;
+
+//  iterator_mapped_type_t
+//
+//  Extracts a mapped type from an iterator.
+template <typename Iter>
+using iterator_mapped_type_t =
+    typename iterator_value_type_t<Iter>::second_type;
+
 /**
  * Argument tuple for variadic emplace/constructor calls. Stores arguments by
  * (decayed) value. Restores original argument types with reference qualifiers
@@ -530,4 +584,216 @@ template <bool implicit_unpack = true, typename Container>
 back_emplace_iterator<Container, implicit_unpack> back_emplacer(Container& c) {
   return back_emplace_iterator<Container, implicit_unpack>(c);
 }
+
+namespace detail {
+
+// An accepted way to make operator-> work
+// https://quuxplusone.github.io/blog/2019/02/06/arrow-proxy/
+template <typename Ref>
+struct arrow_proxy {
+  Ref res;
+  Ref* operator->() { return &res; }
+
+  explicit arrow_proxy(Ref* ref) : res(*ref) {}
+};
+
+} // namespace detail
+
+/**
+ * index_iterator
+ *
+ * An iterator class for random access data structures that provide an
+ * access by index via `operator[](size_type)`.
+ *
+ * Requires a `value_type` defined in a container (we cannot
+ * get the value type from reference).
+ *
+ * Example:
+ *  class Container {
+ *   public:
+ *    using value_type = <*>;  // we need value_type to be defined.
+ *    using iterator = folly::index_iterator<Container>;
+ *    using const_iterator = folly::index_iterator<const Container>;
+ *    using reverse_iterator = std::reverse_iterator<iterator>;
+ *    using const_reverse_iterator = std::reverse_iterator<const_iterator>;
+ *
+ *    some_ref_type  operator[](std::size_t index);
+ *    some_cref_type operator[](std::size_t index) const;
+ *   ...
+ *  };
+ *
+ *  Note that `some_ref_type` can be any proxy reference, as long as the
+ *  algorithms support that (for example from range-v3).
+ *
+ * NOTE: there is no way to override `operator[]`, if that's needed
+ *       we recommend to wrap your data in a struct with `operator[]`.
+ **/
+
+template <typename Container>
+class index_iterator {
+  template <typename T>
+  using get_size_type_t = typename std::remove_cv_t<T>::size_type;
+
+  template <typename T>
+  using get_difference_type_t = typename std::remove_cv_t<T>::difference_type;
+
+ public:
+  // index iterator specific types
+
+  using container_type = Container;
+  using size_type = detected_or_t<std::size_t, get_size_type_t, Container>;
+
+  // iterator types
+
+  using value_type = typename std::remove_const_t<container_type>::value_type;
+  using iterator_category = std::random_access_iterator_tag;
+  using reference = decltype(FOLLY_DECLVAL(container_type&)[size_type{}]);
+  using difference_type =
+      detected_or_t<std::ptrdiff_t, get_difference_type_t, Container>;
+
+  using pointer = std::conditional_t<
+      std::is_reference<reference>::value,
+      std::remove_reference_t<reference>*,
+      detail::arrow_proxy<reference>>;
+
+  static_assert(
+      std::is_signed<difference_type>::value, "difference_type must be signed");
+
+  // accessors
+
+  // instance of `index_iterator_accessor`
+  container_type* get_container() const { return container_; }
+  difference_type get_index() const { return index_; }
+
+  constexpr index_iterator() = default;
+
+  constexpr index_iterator(container_type& container, size_type index)
+      : container_(&container), index_(index) {}
+
+  // converting constructors --
+
+  template <
+      typename OtherContainer,
+      typename = std::enable_if_t<
+          std::is_same<std::remove_const_t<container_type>, OtherContainer>::
+              value &&
+          std::is_const<container_type>::value>>
+  /* implicit */ constexpr index_iterator(index_iterator<OtherContainer> other)
+      : container_(other.get_container()), index_(other.get_index()) {}
+
+  // access ---
+
+  constexpr reference operator*() const { return (*container_)[index_]; }
+
+  pointer operator->() const {
+    // It's equivalent to pointer{&**this} but compiler stops
+    // compilation on taking an address of a temporary.
+    // In this case `arrow_proxy` will copy the temporary and there is no
+    // issue.
+    auto&& ref = **this;
+    pointer res{&ref};
+    return res;
+  }
+
+  constexpr reference operator[](difference_type n) const {
+    return *(*this + n);
+  }
+
+  // operator++/--
+
+  constexpr index_iterator& operator++() {
+    ++index_;
+    return *this;
+  }
+
+  constexpr index_iterator operator++(int) {
+    auto tmp = *this;
+    ++*this;
+    return tmp;
+  }
+
+  constexpr index_iterator& operator--() {
+    --index_;
+    return *this;
+  }
+
+  constexpr index_iterator operator--(int) {
+    auto tmp = *this;
+    --*this;
+    return tmp;
+  }
+
+  // operator+/-
+
+  constexpr index_iterator& operator+=(difference_type n) {
+    auto signed_index = static_cast<difference_type>(index_) + n;
+    index_ = static_cast<size_type>(signed_index);
+    return *this;
+  }
+
+  constexpr index_iterator& operator-=(difference_type n) {
+    index_ += -n;
+    return *this;
+  }
+
+  constexpr friend index_iterator operator+(
+      index_iterator x, difference_type n) {
+    return x += n;
+  }
+
+  constexpr friend index_iterator operator+(
+      difference_type n, index_iterator x) {
+    return x + n;
+  }
+
+  constexpr friend index_iterator operator-(
+      index_iterator x, difference_type n) {
+    return x -= n;
+  }
+
+  constexpr friend difference_type operator-(
+      index_iterator x, index_iterator y) {
+    assert(x.container_ == y.container_);
+    return static_cast<difference_type>(x.index_) -
+        static_cast<difference_type>(y.index_);
+  }
+
+  // comparisons
+  friend constexpr bool operator==(
+      const index_iterator& x, const index_iterator& y) {
+    assert(x.container_ == y.container_);
+    return x.index_ == y.index_;
+  }
+
+  friend constexpr bool operator!=(
+      const index_iterator& x, const index_iterator& y) {
+    return !(x == y);
+  }
+
+  friend constexpr bool operator<(
+      const index_iterator& x, const index_iterator& y) {
+    assert(x.container_ == y.container_);
+    return x.index_ < y.index_;
+  }
+
+  friend constexpr bool operator<=(
+      const index_iterator& x, const index_iterator& y) {
+    return !(y < x);
+  }
+
+  friend constexpr bool operator>=(
+      const index_iterator& x, const index_iterator& y) {
+    return !(x < y);
+  }
+
+  friend constexpr bool operator>(
+      const index_iterator& x, const index_iterator& y) {
+    return y < x;
+  }
+
+ private:
+  container_type* container_ = nullptr;
+  size_type index_ = 0;
+};
+
 } // namespace folly
