@@ -200,30 +200,20 @@ class F14BasicMap : public std::unordered_map<K, M, H, E, A> {
   template <typename... Args>
   std::pair<iterator, bool> emplace(Args&&... args) {
     auto alloc = this->get_allocator();
-    return folly::detail::callWithExtractedKey<
-        key_type,
-        mapped_type,
-        UsableAsKey>(
-        alloc,
-        [&](auto& key, auto&&... inner) {
-          if (!std::is_same<key_type, remove_cvref_t<decltype(key)>>::value) {
-            // this is a heterogeneous lookup
-            auto it = find(key);
-            if (it != this->end()) {
-              return std::make_pair(it, false);
-            }
-            auto rv = Super::emplace(std::forward<decltype(inner)>(inner)...);
-            FOLLY_SAFE_DCHECK(
-                rv.second, "post-find emplace should always insert");
-            return rv;
-          } else {
-            // callWithExtractedKey will use 2 inner args if possible, which
-            // maximizes the changes for the STL emplace to search for an
-            // existing key before constructing a value_type
-            return Super::emplace(std::forward<decltype(inner)>(inner)...);
-          }
-        },
-        std::forward<Args>(args)...);
+    return folly::detail::
+        callWithExtractedKey<key_type, mapped_type, UsableAsKey>(
+            alloc,
+            [&](auto& key, auto&&... inner) {
+              auto it = find(key);
+              if (it != this->end()) {
+                return std::make_pair(it, false);
+              }
+              auto rv = Super::emplace(std::forward<decltype(inner)>(inner)...);
+              FOLLY_SAFE_DCHECK(
+                  rv.second, "post-find emplace should always insert");
+              return rv;
+            },
+            std::forward<Args>(args)...);
   }
 
   template <typename... Args>
@@ -544,6 +534,82 @@ class F14BasicMap : public std::unordered_map<K, M, H, E, A> {
       visitor(b, b + 1);
     }
   }
+
+  /// F14HashToken interface
+  template <typename V>
+  std::pair<iterator, bool> insert_or_assign(
+      F14HashToken const&, key_type const& key, V&& obj) {
+    return insert_or_assign(key, std::forward<V>(obj));
+  }
+
+  template <typename V>
+  std::pair<iterator, bool> insert_or_assign(
+      F14HashToken const&, key_type&& key, V&& obj) {
+    return insert_or_assign(std::move(key), std::forward<V>(obj));
+  }
+
+  template <typename K2, typename V>
+  EnableHeterogeneousInsert<K2, std::pair<iterator, bool>> insert_or_assign(
+      F14HashToken const&, K2&& key, V&& obj) {
+    return insert_or_assign(std::forward<K2>(key), std::forward<V>(obj));
+  }
+
+  template <typename... Args>
+  std::pair<iterator, bool> try_emplace_token(
+      F14HashToken const&, key_type const& key, Args&&... args) {
+    return try_emplace(key, std::forward<Args>(args)...);
+  }
+
+  template <typename... Args>
+  std::pair<iterator, bool> try_emplace_token(
+      F14HashToken const&, key_type&& key, Args&&... args) {
+    return try_emplace(std::move(key), std::forward<Args>(args)...);
+  }
+
+  template <typename K2, typename... Args>
+  EnableHeterogeneousInsert<K2, std::pair<iterator, bool>> try_emplace_token(
+      F14HashToken const&, K2&& key, Args&&... args) {
+    return try_emplace(std::forward<K2>(key), std::forward<Args>(args)...);
+  }
+
+  F14HashToken prehash(key_type const&) const {
+    return {}; // Ignored.
+  }
+
+  template <typename K2>
+  EnableHeterogeneousFind<K2, F14HashToken> prehash(K2 const&) const {
+    return {}; // Ignored.
+  }
+
+  void prefetch(F14HashToken const&) const {}
+
+  iterator find(F14HashToken const&, key_type const& key) { return find(key); }
+
+  const_iterator find(F14HashToken const&, key_type const& key) const {
+    return find(key);
+  }
+
+  template <typename K2>
+  EnableHeterogeneousFind<K2, iterator> find(
+      F14HashToken const&, K2 const& key) {
+    return find(key);
+  }
+
+  template <typename K2>
+  EnableHeterogeneousFind<K2, const_iterator> find(
+      F14HashToken const&, K2 const& key) const {
+    return find(key);
+  }
+
+  bool contains(F14HashToken const&, key_type const& key) const {
+    return contains(key);
+  }
+
+  template <typename K2>
+  EnableHeterogeneousFind<K2, bool> contains(
+      F14HashToken const&, K2 const& key) const {
+    return contains(key);
+  }
 };
 } // namespace detail
 } // namespace f14
