@@ -2225,9 +2225,6 @@ TEST(F14Map, copyAfterRemovedCollisions) {
   testCopyAfterRemovedCollisions<F14FastMap>();
 }
 
-#if FOLLY_HAS_DEDUCTION_GUIDES && /* TODO: Implement deduction guides in \
-                                     fallback implementation. */         \
-    FOLLY_F14_VECTOR_INTRINSICS_AVAILABLE
 template <template <class...> class TMap>
 void testIterDeductionGuide() {
   TMap<int, double> source({{1, 2.0}, {3, 4.0}});
@@ -2350,7 +2347,6 @@ TEST(F14Map, initializerListDeductionGuide) {
   testInitializerListDeductionGuide<F14VectorMap>();
   testInitializerListDeductionGuide<F14FastMap>();
 }
-#endif // FOLLY_HAS_DEDUCTION_GUIDES
 
 namespace {
 
