@@ -1599,9 +1599,6 @@ TEST(F14Set, ExceptionOnInsert) {
   testExceptionOnInsert<F14FastSet>();
 }
 
-#if FOLLY_HAS_DEDUCTION_GUIDES && /* TODO: Implement deduction guides in \
-                                     fallback implementation. */         \
-    FOLLY_F14_VECTOR_INTRINSICS_AVAILABLE
 template <template <class...> class TSet>
 void testIterDeductionGuide() {
   TSet<int> source({1, 2});
@@ -1698,4 +1695,3 @@ TEST(F14Set, initializerListDeductionGuide) {
   testInitializerListDeductionGuide<F14VectorSet>();
   testInitializerListDeductionGuide<F14FastSet>();
 }
-#endif // FOLLY_HAS_DEDUCTION_GUIDES
