@@ -38,6 +38,7 @@
 #include <folly/memory/Malloc.h>
 #include <folly/portability/PThread.h>
 #include <folly/synchronization/MicroSpinLock.h>
+#include <folly/synchronization/RelaxedAtomic.h>
 #include <folly/system/AtFork.h>
 #include <folly/system/ThreadId.h>
 
@@ -127,41 +128,39 @@ struct ElementWrapper {
 
   template <class Ptr>
   void set(Ptr p) {
-    auto guard = makeGuard([&] { delete p; });
     DCHECK(ptr == nullptr);
     DCHECK(deleter1 == nullptr);
 
-    if (p) {
-      node.initIfZero(true /*locked*/);
-      ptr = p;
-      deleter1 = [](void* pt, TLPDestructionMode) {
-        delete static_cast<Ptr>(pt);
-      };
-      ownsDeleter = false;
-      guard.dismiss();
+    if (!p) {
+      return;
     }
+
+    node.initIfZero(true /*locked*/);
+    deleter1 = [](void* pt, TLPDestructionMode) {
+      delete static_cast<Ptr>(pt);
+    };
+    ownsDeleter = false;
+    ptr = p;
   }
 
   template <class Ptr, class Deleter>
   void set(Ptr p, const Deleter& d) {
-    auto guard = makeGuard([&] {
-      if (p) {
-        d(p, TLPDestructionMode::THIS_THREAD);
-      }
-    });
     DCHECK(ptr == nullptr);
     DCHECK(deleter2 == nullptr);
-    if (p) {
-      node.initIfZero(true /*locked*/);
-      ptr = p;
-      auto d2 = d; // gcc-4.8 doesn't decay types correctly in lambda captures
-      deleter2 = new std::function<DeleterFunType>(
-          [d2](void* pt, TLPDestructionMode mode) {
-            d2(static_cast<Ptr>(pt), mode);
-          });
-      ownsDeleter = true;
-      guard.dismiss();
+
+    if (!p) {
+      return;
     }
+
+    node.initIfZero(true /*locked*/);
+    auto guard = makeGuard([&] { d(p, TLPDestructionMode::THIS_THREAD); });
+    deleter2 = new std::function<DeleterFunType>(
+        [d](void* pt, TLPDestructionMode mode) {
+          d(static_cast<Ptr>(pt), mode);
+        });
+    guard.dismiss();
+    ownsDeleter = true;
+    ptr = p;
   }
 
   void cleanup() {
@@ -358,8 +357,8 @@ struct StaticMetaBase {
 
   // push back an entry in the doubly linked list
   // that corresponds to idx id
-  void pushBackLocked(ThreadEntry* t, uint32_t id);
-  void pushBackUnlocked(ThreadEntry* t, uint32_t id);
+  void pushBackLocked(ThreadEntry* t, uint32_t id) noexcept;
+  void pushBackUnlocked(ThreadEntry* t, uint32_t id) noexcept;
 
   // static helper method to reallocate the ThreadEntry::elements
   // returns != nullptr if the ThreadEntry::elements was reallocated
@@ -376,6 +375,11 @@ struct StaticMetaBase {
   ThreadEntry head_;
   ThreadEntry* (*threadEntry_)();
   bool strict_;
+  // Total size of ElementWrapper arrays across all threads. This is meant
+  // to surface the overhead of thread local tracking machinery since the array
+  // can be sparse when there are lots of thread local variables under the same
+  // tag.
+  relaxed_atomic_int64_t totalElementWrappers_{0};
 };
 
 // Held in a singleton to track our global instances.
@@ -427,7 +431,7 @@ struct FOLLY_EXPORT StaticMeta final : StaticMetaBase {
       EntryID* ent, uint32_t& id, ThreadEntry*& threadEntry, size_t& capacity) {
     auto& inst = instance();
     threadEntry = inst.threadEntry_();
-    if (UNLIKELY(threadEntry->getElementsCapacity() <= id)) {
+    if (FOLLY_UNLIKELY(threadEntry->getElementsCapacity() <= id)) {
       inst.reserve(ent);
       id = ent->getOrInvalid();
     }
