@@ -118,7 +118,7 @@ class EDFThreadPoolExecutor::TaskQueue {
     auto deadline = task->getDeadline();
     auto& bucket = getBucket(deadline);
     {
-      SharedMutex::WriteHolder guard(&bucket.mutex);
+      std::unique_lock guard(bucket.mutex);
       task->setEnqueueOrder(bucket.enqueued++);
       bucket.tasks.push(std::move(task));
       bucket.empty.store(bucket.tasks.empty(), std::memory_order_relaxed);
@@ -160,7 +160,7 @@ class EDFThreadPoolExecutor::TaskQueue {
 
       {
         // Fast path. Take bucket reader lock.
-        SharedMutex::ReadHolder guard(&bucket.mutex);
+        std::shared_lock guard(bucket.mutex);
         if (bucket.tasks.empty()) {
           continue;
         }
@@ -173,7 +173,7 @@ class EDFThreadPoolExecutor::TaskQueue {
 
       {
         // Take the writer lock to clean up the finished task.
-        SharedMutex::WriteHolder guard(&bucket.mutex);
+        std::unique_lock guard(bucket.mutex);
         if (bucket.tasks.empty()) {
           continue;
         }
@@ -211,7 +211,7 @@ class EDFThreadPoolExecutor::TaskQueue {
         continue;
       }
 
-      SharedMutex::ReadHolder guard(&bucket.mutex);
+      std::shared_lock guard(bucket.mutex);
       auto curDeadline = curDeadline_.load(std::memory_order_relaxed);
       if (prevDeadline != curDeadline) {
         // Bail out early if something already happened
@@ -285,7 +285,7 @@ void EDFThreadPoolExecutor::add(Func f) {
 }
 
 void EDFThreadPoolExecutor::add(Func f, std::size_t total, uint64_t deadline) {
-  if (UNLIKELY(isJoin_.load(std::memory_order_relaxed) || total == 0)) {
+  if (FOLLY_UNLIKELY(isJoin_.load(std::memory_order_relaxed) || total == 0)) {
     return;
   }
 
@@ -300,7 +300,7 @@ void EDFThreadPoolExecutor::add(Func f, std::size_t total, uint64_t deadline) {
 }
 
 void EDFThreadPoolExecutor::add(std::vector<Func> fs, uint64_t deadline) {
-  if (UNLIKELY(fs.empty())) {
+  if (FOLLY_UNLIKELY(fs.empty())) {
     return;
   }
 
@@ -325,9 +325,9 @@ void EDFThreadPoolExecutor::threadRun(ThreadPtr thread) {
     auto task = take();
 
     // Handle thread stopping
-    if (UNLIKELY(!task)) {
+    if (FOLLY_UNLIKELY(!task)) {
       // Actually remove the thread from the list.
-      SharedMutex::WriteHolder w{&threadListLock_};
+      std::unique_lock w{threadListLock_};
       for (auto& o : observers_) {
         o->threadStopped(thread.get());
       }
@@ -337,7 +337,7 @@ void EDFThreadPoolExecutor::threadRun(ThreadPtr thread) {
     }
 
     int iter = task->next();
-    if (UNLIKELY(iter < 0)) {
+    if (FOLLY_UNLIKELY(iter < 0)) {
       // This task is already finished
       continue;
     }
@@ -398,7 +398,7 @@ bool EDFThreadPoolExecutor::shouldStop() {
 }
 
 std::shared_ptr<EDFThreadPoolExecutor::Task> EDFThreadPoolExecutor::take() {
-  if (UNLIKELY(shouldStop())) {
+  if (FOLLY_UNLIKELY(shouldStop())) {
     return nullptr;
   }
 
@@ -406,7 +406,7 @@ std::shared_ptr<EDFThreadPoolExecutor::Task> EDFThreadPoolExecutor::take() {
     return task;
   }
 
-  if (UNLIKELY(isJoin_.load(std::memory_order_relaxed))) {
+  if (FOLLY_UNLIKELY(isJoin_.load(std::memory_order_relaxed))) {
     return nullptr;
   }
 
@@ -416,7 +416,7 @@ std::shared_ptr<EDFThreadPoolExecutor::Task> EDFThreadPoolExecutor::take() {
   SCOPE_EXIT { numIdleThreads_.fetch_sub(1, std::memory_order_seq_cst); };
 
   for (;;) {
-    if (UNLIKELY(shouldStop())) {
+    if (FOLLY_UNLIKELY(shouldStop())) {
       return nullptr;
     }
 
@@ -426,7 +426,7 @@ std::shared_ptr<EDFThreadPoolExecutor::Task> EDFThreadPoolExecutor::take() {
       return task;
     }
 
-    if (UNLIKELY(isJoin_.load(std::memory_order_relaxed))) {
+    if (FOLLY_UNLIKELY(isJoin_.load(std::memory_order_relaxed))) {
       return nullptr;
     }
 
