@@ -39,6 +39,7 @@
 #include <folly/executors/thread_factory/PriorityThreadFactory.h>
 #include <folly/portability/GTest.h>
 #include <folly/portability/PThread.h>
+#include <folly/portability/SysResource.h>
 #include <folly/synchronization/detail/Spin.h>
 
 using namespace folly;
@@ -1113,6 +1114,7 @@ class SingleThreadedCPUThreadPoolExecutor : public CPUThreadPoolExecutor,
  public:
   explicit SingleThreadedCPUThreadPoolExecutor(size_t)
       : CPUThreadPoolExecutor(1) {}
+  ~SingleThreadedCPUThreadPoolExecutor() override { stop(); }
 };
 
 TYPED_TEST(ThreadPoolExecutorTypedTest, WeakRef) {
