@@ -60,15 +60,25 @@ Optional<SettingMetadata> getSettingsMeta(StringPiece settingName) {
   return it->second->meta();
 }
 
-bool Snapshot::setFromString(
+SetResult Snapshot::setFromString(
     StringPiece settingName, StringPiece newValue, StringPiece reason) {
   auto mapPtr = detail::settingsMap().rlock();
   auto it = mapPtr->find(settingName.str());
   if (it == mapPtr->end()) {
-    return false;
+    return makeUnexpected(SetErrorCode::NotFound);
   }
-  it->second->setFromString(newValue, reason, this);
-  return true;
+  return it->second->setFromString(newValue, reason, this);
+}
+
+SetResult Snapshot::forceSetFromString(
+    StringPiece settingName, StringPiece newValue, StringPiece reason) {
+  auto mapPtr = detail::settingsMap().rlock();
+  auto it = mapPtr->find(settingName.str());
+  if (it == mapPtr->end()) {
+    return makeUnexpected(SetErrorCode::NotFound);
+  }
+  it->second->forceSetFromString(newValue, reason, this);
+  return folly::unit;
 }
 
 Optional<Snapshot::SettingsInfo> Snapshot::getAsString(
@@ -81,14 +91,23 @@ Optional<Snapshot::SettingsInfo> Snapshot::getAsString(
   return it->second->getAsString(this);
 }
 
-bool Snapshot::resetToDefault(StringPiece settingName) {
+SetResult Snapshot::resetToDefault(StringPiece settingName) {
   auto mapPtr = detail::settingsMap().rlock();
   auto it = mapPtr->find(settingName.str());
   if (it == mapPtr->end()) {
-    return false;
+    return makeUnexpected(SetErrorCode::NotFound);
   }
-  it->second->resetToDefault(this);
-  return true;
+  return it->second->resetToDefault(this);
+}
+
+SetResult Snapshot::forceResetToDefault(StringPiece settingName) {
+  auto mapPtr = detail::settingsMap().rlock();
+  auto it = mapPtr->find(settingName.str());
+  if (it == mapPtr->end()) {
+    return makeUnexpected(SetErrorCode::NotFound);
+  }
+  it->second->forceResetToDefault(this);
+  return folly::unit;
 }
 
 void Snapshot::forEachSetting(
@@ -173,8 +192,11 @@ SnapshotBase::~SnapshotBase() {
 } // namespace detail
 
 void Snapshot::publish() {
+  // Double check frozen immutables since they could have been frozen after the
+  // values were set.
+  auto frozenProjects = frozenSettingProjects();
   for (auto& it : snapshotValues_) {
-    it.second.publish();
+    it.second.publish(frozenProjects);
   }
 }
 
