@@ -17,6 +17,8 @@
 #include <folly/experimental/settings/Settings.h>
 
 #include <folly/Format.h>
+#include <folly/String.h>
+#include <folly/portability/GMock.h>
 #include <folly/portability/GTest.h>
 
 #include <folly/experimental/settings/test/a.h>
@@ -24,24 +26,35 @@
 
 namespace some_ns {
 FOLLY_SETTING_DEFINE(
-    follytest, some_flag, std::string, "default", "Description");
+    follytest,
+    some_flag,
+    std::string,
+    "default",
+    folly::settings::Mutability::Mutable,
+    "Description");
 FOLLY_SETTING_DEFINE(
     follytest,
     unused,
     std::string,
     "unused_default",
+    folly::settings::Mutability::Mutable,
     "Not used, but should still be in the list");
 FOLLY_SETTING_DEFINE(
     follytest,
     multi_token_type,
     unsigned int,
     123,
+    folly::settings::Mutability::Mutable,
     "Test that multi-token type names can be used");
 // Enable to test runtime collision checking logic
 #if 0
-FOLLY_SETTING_DEFINE(follytest, internal_flag_to_a, std::string,
-                     "collision_with_a",
-                     "Collision_with_a");
+FOLLY_SETTING_DEFINE(
+    follytest,
+    internal_flag_to_a,
+    std::string,
+    "collision_with_a",
+    folly::settings::Mutability::Mutable,
+    "Collision_with_a");
 #endif
 
 /* Test user defined type support */
@@ -80,15 +93,56 @@ FOLLY_SETTING_DEFINE(
     user_defined,
     UserDefinedType,
     "b",
+    folly::settings::Mutability::Mutable,
     "User defined type constructed from string");
 
+FOLLY_SETTING_DEFINE(
+    follytest,
+    immutable_setting,
+    UserDefinedType,
+    "b",
+    folly::settings::Mutability::Immutable,
+    "User defined type constructed from string");
+FOLLY_SETTING_DEFINE(
+    otherproj,
+    some_flag,
+    std::string,
+    "default",
+    folly::settings::Mutability::Mutable,
+    "Description");
+
 } // namespace some_ns
+namespace {
+MATCHER(IsOk, "success but got error") {
+  if (arg.hasError()) {
+    *result_listener << "which is: '"
+                     << ::folly::settings::toString(arg.error()) << "'";
+    return false;
+  }
+  return true;
+}
+MATCHER_P(
+    HasErrorCode,
+    error_code,
+    "error '" + std::string(::folly::settings::toString(error_code)) + "'") {
+  if (!arg.hasError()) {
+    *result_listener << "which is not an error";
+    return false;
+  }
+  if (arg.error() != error_code) {
+    *result_listener << "which is: '"
+                     << ::folly::settings::toString(arg.error()) << "'";
+  }
+  return true;
+}
+} // namespace
 
 TEST(Settings, userDefined) {
   EXPECT_EQ(some_ns::FOLLY_SETTING(follytest, user_defined)->value_, 100);
   {
     folly::settings::Snapshot sn;
-    EXPECT_TRUE(sn.setFromString("follytest_user_defined", "a", "test"));
+    EXPECT_THAT(
+        sn.setFromString("follytest_user_defined", "a", "test"), IsOk());
     sn.publish();
     EXPECT_EQ(some_ns::FOLLY_SETTING(follytest, user_defined)->value_, 0);
   }
@@ -116,7 +170,7 @@ TEST(Settings, userDefined) {
   }
   {
     folly::settings::Snapshot sn;
-    EXPECT_TRUE(sn.resetToDefault("follytest_user_defined"));
+    EXPECT_THAT(sn.resetToDefault("follytest_user_defined"), IsOk());
     sn.publish();
     EXPECT_EQ(some_ns::FOLLY_SETTING(follytest, user_defined)->value_, 100);
   }
@@ -187,8 +241,9 @@ TEST(Settings, basic) {
   }
   {
     folly::settings::Snapshot sn;
-    EXPECT_TRUE(
-        sn.setFromString("follytest_public_flag_to_a", "300", "from_string"));
+    EXPECT_THAT(
+        sn.setFromString("follytest_public_flag_to_a", "300", "from_string"),
+        IsOk());
     sn.publish();
     EXPECT_EQ(*a_ns::FOLLY_SETTING(follytest, public_flag_to_a), 300);
   }
@@ -202,8 +257,9 @@ TEST(Settings, basic) {
   }
   {
     folly::settings::Snapshot sn;
-    EXPECT_FALSE(
-        sn.setFromString("follytest_nonexisting", "300", "from_string"));
+    EXPECT_THAT(
+        sn.setFromString("follytest_nonexisting", "300", "from_string"),
+        HasErrorCode(folly::settings::SetErrorCode::NotFound));
   }
   EXPECT_EQ(
       some_ns::FOLLY_SETTING(follytest, multi_token_type).defaultValue(), 123);
@@ -244,27 +300,32 @@ TEST(Settings, basic) {
           value,
           reason);
     });
-    EXPECT_EQ(
-        allFlags,
-        "follytest/internal_flag_to_a/int/789/Desc of int/789/default\n"
-        "follytest/internal_flag_to_b/std::string/\"test\"/Desc of str/test/default\n"
-        "follytest/multi_token_type/unsigned int/123/Test that multi-token type names can be used/123/default\n"
-        "follytest/public_flag_to_a/int/456/Public flag to a/300/from_string\n"
-        "follytest/public_flag_to_b/std::string/\"basdf\"/Public flag to b/basdf/default\n"
-        "follytest/some_flag/std::string/\"default\"/Description/default/default\n"
-        "follytest/unused/std::string/\"unused_default\"/Not used, but should still be in the list/unused_default/default\n"
-        "follytest/user_defined/UserDefinedType/\"b\"/User defined type constructed from string/b_out/default\n");
+    auto allFlagsString = folly::stripLeftMargin(R"MESSAGE(
+      follytest/immutable_setting/UserDefinedType/"b"/User defined type constructed from string/b_out/default
+      follytest/internal_flag_to_a/int/789/Desc of int/789/default
+      follytest/internal_flag_to_b/std::string/"test"/Desc of str/test/default
+      follytest/multi_token_type/unsigned int/123/Test that multi-token type names can be used/123/default
+      follytest/public_flag_to_a/int/456/Public flag to a/300/from_string
+      follytest/public_flag_to_b/std::string/"basdf"/Public flag to b/basdf/default
+      follytest/some_flag/std::string/"default"/Description/default/default
+      follytest/unused/std::string/"unused_default"/Not used, but should still be in the list/unused_default/default
+      follytest/user_defined/UserDefinedType/"b"/User defined type constructed from string/b_out/default
+      otherproj/some_flag/std::string/"default"/Description/default/default
+  )MESSAGE");
+    EXPECT_EQ(allFlags, allFlagsString);
   }
   {
     folly::settings::Snapshot sn;
-    EXPECT_TRUE(sn.resetToDefault("follytest_public_flag_to_a"));
+    EXPECT_THAT(sn.resetToDefault("follytest_public_flag_to_a"), IsOk());
     sn.publish();
     EXPECT_EQ(*a_ns::FOLLY_SETTING(follytest, public_flag_to_a), 456);
     EXPECT_EQ(a_ns::getRemote(), 456);
   }
   {
     folly::settings::Snapshot sn;
-    EXPECT_FALSE(sn.resetToDefault("follytest_nonexisting"));
+    EXPECT_THAT(
+        sn.resetToDefault("follytest_nonexisting"),
+        HasErrorCode(folly::settings::SetErrorCode::NotFound));
   }
 }
 
@@ -490,3 +551,125 @@ TEST(SettingsTest, callback) {
   EXPECT_EQ(callbackInvocations, 4);
   EXPECT_EQ(lastCallbackValue, "d");
 }
+
+TEST(Settings, immutables) {
+  EXPECT_EQ(some_ns::FOLLY_SETTING(follytest, immutable_setting)->value_, 100);
+  EXPECT_EQ(*some_ns::FOLLY_SETTING(otherproj, some_flag), "default");
+
+  // Check these can be changed before immutables are frozen
+  folly::settings::Snapshot sn;
+  EXPECT_THAT(
+      sn.setFromString("follytest_immutable_setting", "a", "test"), IsOk());
+  EXPECT_THAT(
+      sn.setFromString("otherproj_some_flag", "value_1", "test"), IsOk());
+  sn.publish();
+  EXPECT_EQ(some_ns::FOLLY_SETTING(follytest, immutable_setting)->value_, 0);
+  EXPECT_EQ(*some_ns::FOLLY_SETTING(otherproj, some_flag), "value_1");
+
+  // Create a snapshot before freezing immutables
+  folly::settings::Snapshot beforeFreezeSn;
+  EXPECT_THAT(
+      beforeFreezeSn.resetToDefault("follytest_immutable_setting"), IsOk());
+  EXPECT_THAT(
+      beforeFreezeSn.setFromString("follytest_immutable_setting", "b", "test"),
+      IsOk());
+  EXPECT_THAT(
+      beforeFreezeSn.setFromString("otherproj_some_flag", "value_2", "test"),
+      IsOk());
+
+  ASSERT_FALSE(folly::settings::immutablesFrozen("follytest"));
+  ASSERT_FALSE(folly::settings::frozenSettingProjects().contains("follytest"));
+
+  // Freeze "follytest" immutable settings
+  folly::settings::freezeImmutables({"follytest"});
+
+  ASSERT_TRUE(folly::settings::immutablesFrozen("follytest"));
+  ASSERT_TRUE(folly::settings::frozenSettingProjects().contains("follytest"));
+  ASSERT_FALSE(folly::settings::immutablesFrozen("otherproj"));
+  ASSERT_FALSE(folly::settings::frozenSettingProjects().contains("otherproj"));
+
+  // Check that even though 'immutable_setting' was successfully set before
+  // settings were frozen, it still can't publish the new value now that
+  // immutables are frozen. However, it still can change otherproj_some_flag
+  // since that project has not been frozen yet.
+  beforeFreezeSn.publish();
+  EXPECT_EQ(some_ns::FOLLY_SETTING(follytest, immutable_setting)->value_, 0);
+  EXPECT_EQ(*some_ns::FOLLY_SETTING(otherproj, some_flag), "value_2");
+
+  // Check newly created snapshots fail mutation attempts.
+  folly::settings::Snapshot afterFreezeSn;
+  EXPECT_THAT(
+      afterFreezeSn.setFromString("follytest_immutable_setting", "b", "test"),
+      HasErrorCode(folly::settings::SetErrorCode::FrozenImmutable));
+  EXPECT_THAT(
+      afterFreezeSn.resetToDefault("follytest_immutable_setting"),
+      HasErrorCode(folly::settings::SetErrorCode::FrozenImmutable));
+  EXPECT_THAT(
+      some_ns::FOLLY_SETTING(follytest, immutable_setting)
+          .set(some_ns::UserDefinedType("b")),
+      HasErrorCode(folly::settings::SetErrorCode::FrozenImmutable));
+
+  // Check we don't attempt to contruct immutable settings from strings if
+  // immutables have already been frozen. If we did, the below code would throw
+  // an exception.
+  some_ns::UserDefinedType bad("a");
+  bad.value_ = 50;
+  EXPECT_THAT(
+      some_ns::FOLLY_SETTING(follytest, immutable_setting).set(bad),
+      HasErrorCode(folly::settings::SetErrorCode::FrozenImmutable));
+  EXPECT_THAT(
+      afterFreezeSn.setFromString("follytest_immutable_setting", "c", "test"),
+      HasErrorCode(folly::settings::SetErrorCode::FrozenImmutable));
+
+  // Check forceSetFromString does construct immutable settings even if
+  // immutables have already been frozen.
+  try {
+    afterFreezeSn.forceSetFromString("follytest_immutable_setting", "c", "ds");
+    FAIL();
+  } catch (const std::runtime_error& ex) {
+    EXPECT_STREQ(ex.what(), "Invalid value passed to UserDefinedType ctor");
+  }
+
+  {
+    // Check forceSetFromString does construct immutable settings even if
+    // immutables have already been frozen.
+    folly::settings::Snapshot newSnapshot;
+    try {
+      newSnapshot.forceSetFromString("follytest_immutable_setting", "c", "dsc");
+      FAIL();
+    } catch (const std::runtime_error& ex) {
+      EXPECT_STREQ(ex.what(), "Invalid value passed to UserDefinedType ctor");
+    }
+
+    // Check forceSetFromString can set immutable settings in the snapshot but
+    // will still not publish them
+    EXPECT_THAT(
+        newSnapshot.forceSetFromString(
+            "follytest_immutable_setting", "b", "new desc"),
+        IsOk());
+    auto found = newSnapshot.getAsString("follytest_immutable_setting");
+    ASSERT_TRUE(found.hasValue());
+    EXPECT_EQ(found->first, "b_out");
+    EXPECT_EQ(found->second, "new desc");
+    newSnapshot.publish();
+    EXPECT_EQ(some_ns::FOLLY_SETTING(follytest, immutable_setting)->value_, 0);
+  }
+
+  {
+    // Check forceResetToDefault can reset immutable settings in the snapshot
+    // but will still not publish them
+    folly::settings::Snapshot newSnapshot;
+    auto found = newSnapshot.getAsString("follytest_immutable_setting");
+    ASSERT_TRUE(found.hasValue());
+    EXPECT_EQ(found->first, "a_out");
+    EXPECT_EQ(found->second, "test");
+    EXPECT_THAT(
+        newSnapshot.forceResetToDefault("follytest_immutable_setting"), IsOk());
+    found = newSnapshot.getAsString("follytest_immutable_setting");
+    ASSERT_TRUE(found.hasValue());
+    EXPECT_EQ(found->first, "b_out");
+    EXPECT_EQ(found->second, "default");
+    newSnapshot.publish();
+    EXPECT_EQ(some_ns::FOLLY_SETTING(follytest, immutable_setting)->value_, 0);
+  }
+}
