@@ -16,6 +16,7 @@
 
 #include <array>
 #include <atomic>
+#include <chrono>
 #include <thread>
 #include <vector>
 
@@ -211,6 +212,82 @@ TEST(FiberManager, fiberTimeLogged) {
   manager.loopUntilNoReady();
 }
 
+#ifdef __linux__
+
+namespace {
+void burnCpu(std::chrono::milliseconds duration) {
+  auto expires = folly::fibers::thread_clock::now() + duration;
+  while (folly::fibers::thread_clock::now() < expires) {
+  }
+}
+
+void sleepFor(std::chrono::milliseconds duration) {
+  /* sleep override */ std::this_thread::sleep_for(duration);
+}
+} // namespace
+
+TEST(FiberManager, fiberTimeWhileRunning) {
+  using namespace std::chrono;
+  FiberManager fm(std::make_unique<SimpleLoopController>());
+  TaskOptions tOpt;
+  tOpt.logRunningTime = true;
+  fm.addTask(
+      [&]() {
+        burnCpu(200ms);
+        sleepFor(1s); // (not included in running time)
+        fm.yield();
+        burnCpu(100ms);
+        auto dur = fm.getCurrentTaskRunningTime();
+        ASSERT_TRUE(dur); // ~300ms
+        EXPECT_GE(*dur, 250ms);
+        EXPECT_LE(*dur, 350ms);
+      },
+      tOpt);
+  fm.addTask(
+      [&]() {
+        burnCpu(300ms);
+        sleepFor(1s); // (not included in running time)
+        fm.yield();
+        burnCpu(200ms);
+        auto dur = fm.getCurrentTaskRunningTime();
+        ASSERT_TRUE(dur); // ~500ms
+        EXPECT_GE(*dur, 450ms);
+        EXPECT_LE(*dur, 550ms);
+      },
+      tOpt);
+  while (fm.hasTasks()) {
+    fm.loopUntilNoReady();
+  }
+}
+
+TEST(FiberManager, fiberTimeWhileAwaiting) {
+  using namespace std::chrono;
+  FiberManager fm(std::make_unique<SimpleLoopController>());
+  TaskOptions tOpt;
+  tOpt.logRunningTime = true;
+  fm.addTask(
+      [&]() {
+        burnCpu(200ms);
+        sleepFor(1s); // (not included in running time)
+        fm.yield();
+        burnCpu(100ms);
+        runInMainContext([&] {
+          burnCpu(200ms); // (not included in running time)
+          ASSERT_TRUE(fm.currentFiber());
+          auto dur = fm.getCurrentTaskRunningTime();
+          ASSERT_TRUE(dur); // ~300ms
+          EXPECT_GE(*dur, 250ms);
+          EXPECT_LE(*dur, 350ms);
+        });
+      },
+      tOpt);
+  while (fm.hasTasks()) {
+    fm.loopUntilNoReady();
+  }
+}
+
+#endif // __linux__
+
 TEST(FiberManager, batonTryWait) {
   FiberManager manager(std::make_unique<SimpleLoopController>());
 
@@ -2296,7 +2373,7 @@ void validateResults(
   results.reserve(COUNT);                                       \
   DispatchFunctionT dispatchFunc
 
-TEST(FiberManager, ABD_Test) {
+TEST(FiberManager, ABDTest) {
   SET_UP_TEST_FUNC;
 
   //
@@ -2312,7 +2389,7 @@ TEST(FiberManager, ABD_Test) {
   validateResults(results, COUNT);
 }
 
-TEST(FiberManager, ABD_DispatcherDestroyedBeforeCallingCommit) {
+TEST(FiberManager, ABDDispatcherdestroyedbeforecallingcommit) {
   SET_UP_TEST_FUNC;
 
   //
@@ -2337,7 +2414,7 @@ TEST(FiberManager, ABD_DispatcherDestroyedBeforeCallingCommit) {
   validateResults<ABDCommitNotCalledException>(results, COUNT);
 }
 
-TEST(FiberManager, ABD_PreprocessingFailureTest) {
+TEST(FiberManager, ABDPreprocessingfailuretest) {
   SET_UP_TEST_FUNC;
 
   //
@@ -2353,7 +2430,7 @@ TEST(FiberManager, ABD_PreprocessingFailureTest) {
   validateResults<ABDTokenNotDispatchedException>(results, COUNT - 1);
 }
 
-TEST(FiberManager, ABD_MultipleDispatchOnSameTokenErrorTest) {
+TEST(FiberManager, ABDMultipledispatchonsametokenerrortest) {
   SET_UP_TEST_FUNC;
 
   //
@@ -2368,7 +2445,7 @@ TEST(FiberManager, ABD_MultipleDispatchOnSameTokenErrorTest) {
   evb.loop();
 }
 
-TEST(FiberManager, ABD_GetTokenCalledAfterCommitTest) {
+TEST(FiberManager, ABDGettokencalledaftercommittest) {
   SET_UP_TEST_FUNC;
 
   //
@@ -2387,7 +2464,7 @@ TEST(FiberManager, ABD_GetTokenCalledAfterCommitTest) {
   EXPECT_THROW(atomicBatchDispatcher.getToken(), ABDUsageException);
 }
 
-TEST(FiberManager, ABD_UserProvidedBatchDispatchThrowsTest) {
+TEST(FiberManager, ABDUserprovidedbatchdispatchthrowstest) {
   SET_UP_TEST_FUNC;
 
   //
@@ -2890,3 +2967,19 @@ TEST(FiberManager, fibersPreserveAsyncStackRoots) {
     EXPECT_EQ(originalRoot, folly::tryGetCurrentAsyncStackRoot());
   }
 }
+
+TEST(FiberManager, EventBaseMigratingThreads) {
+  folly::EventBase evb;
+  auto& fm = getFiberManager(evb);
+  folly::fibers::Baton baton;
+  auto f = fm.addTaskFuture([&baton] { baton.wait(); });
+  evb.drive();
+
+  std::thread anotherThread([&] { evb.drive(); });
+
+  std::this_thread::sleep_for(std::chrono::milliseconds{100});
+  baton.post();
+  anotherThread.join();
+
+  EXPECT_TRUE(f.isReady());
+}
