@@ -47,6 +47,15 @@ compiler_may_unsafely_assume_unreachable() {
 #endif
 }
 
+FOLLY_ALWAYS_INLINE void compiler_may_unsafely_assume_separate_storage(
+    void const* const a, void const* const b) {
+  FOLLY_SAFE_DCHECK(
+      a != b, "compiler-hint separate storage assumption fails at runtime");
+#if FOLLY_HAS_BUILTIN(__builtin_assume_separate_storage)
+  __builtin_assume_separate_storage(a, b);
+#endif
+}
+
 #if defined(_MSC_VER) && !defined(__clang__)
 
 namespace detail {
@@ -130,17 +139,6 @@ FOLLY_ALWAYS_INLINE void compiler_must_not_predict_fn::operator()(
   detail::compiler_must_not_predict(t, i{});
 }
 
-FOLLY_ALWAYS_INLINE void compiler_may_assume_separate_storage(
-    const void* a, const void* b) {
-  FOLLY_SAFE_DCHECK(
-      a != b, "compiler-hint separate storage assumption fails at runtime");
-#if defined(__has_builtin)
-#if __has_builtin(__builtin_assume_separate_storage)
-  __builtin_assume_separate_storage(a, b);
-#endif
-#endif
-}
-
 #endif
 
 } // namespace folly
