@@ -29,12 +29,18 @@ namespace detail {
 template <std::size_t S>
 using pretty_carray = c_array<char, S>;
 
+static constexpr char* pretty_carray_copy(
+    char* dest, const char* src, std::size_t n) {
+  for (std::size_t i = 0; i < n; ++i) {
+    dest[i] = src[i];
+  }
+  return dest + n;
+}
+
 template <std::size_t S>
 static constexpr pretty_carray<S> pretty_carray_from(char const (&in)[S]) {
   pretty_carray<S> out{};
-  for (std::size_t i = 0; i < S; ++i) {
-    out.data[i] = in[i];
-  }
+  pretty_carray_copy(out.data, in, S);
   return out;
 }
 
@@ -80,7 +86,7 @@ using pretty_default_tag = std::conditional_t< //
 
 template <typename T>
 static constexpr auto pretty_raw(pretty_tag_msc) {
-#if defined(_MSC_VER)
+#if defined(_MSC_VER) && !defined(__clang__)
   return pretty_carray_from(__FUNCSIG__);
 #endif
 }
@@ -123,9 +129,7 @@ struct pretty_name_zarray {
   static constexpr auto size = info.e - info.b;
   static constexpr auto zarray_() {
     pretty_carray<size + 1> data{};
-    for (std::size_t i = 0; i < size; ++i) {
-      data.data[i] = raw.data[info.b + i];
-    }
+    pretty_carray_copy(data.data, raw.data + info.b, size);
     data.data[size] = 0;
     return data;
   }
@@ -138,6 +142,11 @@ constexpr pretty_carray<pretty_name_zarray<T, Tag>::size + 1>
     pretty_name_zarray<T, Tag>::zarray;
 #endif
 
+template <typename T>
+constexpr const auto& pretty_name_carray() {
+  return detail::pretty_name_zarray<T, detail::pretty_default_tag>::zarray;
+}
+
 } // namespace detail
 
 //  pretty_name
@@ -150,7 +159,7 @@ constexpr pretty_carray<pretty_name_zarray<T, Tag>::size + 1>
 //  present in the type name as it would be symbolized.
 template <typename T>
 constexpr char const* pretty_name() {
-  return detail::pretty_name_zarray<T, detail::pretty_default_tag>::zarray.data;
+  return detail::pretty_name_carray<T>().data;
 }
 
 } // namespace folly
