@@ -154,23 +154,51 @@ static_assert(
           }(),                                                               \
       ##__VA_ARGS__)
 
+/**
+ * Similar to XLOG(...) except log a message if the specified condition
+ * predicate evaluates to true or every @param ms milliseconds
+ *
+ * Note that this is threadsafe.
+ */
+#define XLOG_EVERY_MS_OR(level, cond, ms, ...)                               \
+  XLOG_IF(                                                                   \
+      level,                                                                 \
+      (cond) ||                                                              \
+          [__folly_detail_xlog_ms = ms] {                                    \
+            static ::folly::logging::IntervalRateLimiter                     \
+                folly_detail_xlog_limiter(                                   \
+                    1, ::std::chrono::milliseconds(__folly_detail_xlog_ms)); \
+            return folly_detail_xlog_limiter.check();                        \
+          }(),                                                               \
+      ##__VA_ARGS__)
+
+/**
+ * Similar to XLOGF(...) except only log a message every @param ms
+ * milliseconds and if the specified condition predicate evaluates to true.
+ *
+ * Note that this is threadsafe.
+ */
+#define XLOGF_EVERY_MS_IF(level, cond, ms, fmt, ...)                         \
+  XLOGF_IF(                                                                  \
+      level,                                                                 \
+      (cond) &&                                                              \
+          [__folly_detail_xlog_ms = ms] {                                    \
+            static ::folly::logging::IntervalRateLimiter                     \
+                folly_detail_xlog_limiter(                                   \
+                    1, ::std::chrono::milliseconds(__folly_detail_xlog_ms)); \
+            return folly_detail_xlog_limiter.check();                        \
+          }(),                                                               \
+      fmt,                                                                   \
+      ##__VA_ARGS__)
+
 /**
  * Similar to XLOGF(...) except only log a message every @param ms
  * milliseconds.
  *
  * Note that this is threadsafe.
  */
-#define XLOGF_EVERY_MS(level, ms, fmt, ...)                              \
-  XLOGF_IF(                                                              \
-      level,                                                             \
-      [__folly_detail_xlog_ms = ms] {                                    \
-        static ::folly::logging::IntervalRateLimiter                     \
-            folly_detail_xlog_limiter(                                   \
-                1, ::std::chrono::milliseconds(__folly_detail_xlog_ms)); \
-        return folly_detail_xlog_limiter.check();                        \
-      }(),                                                               \
-      fmt,                                                               \
-      ##__VA_ARGS__)
+#define XLOGF_EVERY_MS(level, ms, fmt, ...) \
+  XLOGF_EVERY_MS_IF(level, true, ms, fmt, ##__VA_ARGS__)
 
 namespace folly {
 namespace detail {
@@ -205,6 +233,26 @@ FOLLY_EXPORT FOLLY_ALWAYS_INLINE bool xlogEveryNImpl(size_t n) {
       }(),                                                                \
       ##__VA_ARGS__)
 
+/**
+ * Similar to XLOGF(...) except only log a message every @param n
+ * invocations, approximately.
+ *
+ * The internal counter is process-global and threadsafe but, to
+ * to avoid the performance degradation of atomic-rmw operations,
+ * increments are non-atomic. Some increments may be missed under
+ * contention, leading to possible over-logging or under-logging
+ * effects.
+ */
+#define XLOGF_EVERY_N(level, n, fmt, ...)                                 \
+  XLOGF_IF(                                                               \
+      level,                                                              \
+      [&] {                                                               \
+        struct folly_detail_xlog_tag {};                                  \
+        return ::folly::detail::xlogEveryNImpl<folly_detail_xlog_tag>(n); \
+      }(),                                                                \
+      fmt,                                                                \
+      ##__VA_ARGS__)
+
 /**
  * Similar to XLOG(...) except only log a message every @param n
  * invocations, approximately, and if the specified condition predicate
@@ -226,6 +274,48 @@ FOLLY_EXPORT FOLLY_ALWAYS_INLINE bool xlogEveryNImpl(size_t n) {
           }(),                                                                \
       ##__VA_ARGS__)
 
+/**
+ * Similar to XLOG(...) except it logs a message if the condition predicate
+ * evalutes to true or approximately every @param n invocations
+ *
+ * The internal counter is process-global and threadsafe but, to
+ * to avoid the performance degradation of atomic-rmw operations,
+ * increments are non-atomic. Some increments may be missed under
+ * contention, leading to possible over-logging or under-logging
+ * effects.
+ */
+#define XLOG_EVERY_N_OR(level, cond, n, ...)                                  \
+  XLOG_IF(                                                                    \
+      level,                                                                  \
+      (cond) ||                                                               \
+          [&] {                                                               \
+            struct folly_detail_xlog_tag {};                                  \
+            return ::folly::detail::xlogEveryNImpl<folly_detail_xlog_tag>(n); \
+          }(),                                                                \
+      ##__VA_ARGS__)
+
+/**
+ * Similar to XLOGF(...) except only log a message every @param n
+ * invocations, approximately, and if the specified condition predicate
+ * evaluates to true.
+ *
+ * The internal counter is process-global and threadsafe but, to
+ * to avoid the performance degradation of atomic-rmw operations,
+ * increments are non-atomic. Some increments may be missed under
+ * contention, leading to possible over-logging or under-logging
+ * effects.
+ */
+#define XLOGF_EVERY_N_IF(level, cond, n, fmt, ...)                            \
+  XLOGF_IF(                                                                   \
+      level,                                                                  \
+      (cond) &&                                                               \
+          [&] {                                                               \
+            struct folly_detail_xlog_tag {};                                  \
+            return ::folly::detail::xlogEveryNImpl<folly_detail_xlog_tag>(n); \
+          }(),                                                                \
+      fmt,                                                                    \
+      ##__VA_ARGS__)
+
 namespace folly {
 namespace detail {
 
@@ -387,7 +477,7 @@ FOLLY_EXPORT FOLLY_ALWAYS_INLINE bool xlogFirstNExactImpl(std::size_t n) {
  *   requirements of being a single expression, but fortunately static
  *   variables inside a lambda work for this purpose.
  *
- *   Inside header files, each XLOG() statement defines to static variables:
+ *   Inside header files, each XLOG() statement defines two static variables:
  *   - the LogLevel for this category
  *   - a pointer to the LogCategory
  *
@@ -571,8 +661,12 @@ FOLLY_EXPORT FOLLY_ALWAYS_INLINE bool xlogFirstNExactImpl(std::size_t n) {
  * false.  Unlike assert() CHECK statements are always enabled, regardless of
  * the setting of NDEBUG.
  */
-#define XCHECK(cond, ...) \
-  XLOG_IF(FATAL, UNLIKELY(!(cond)), "Check failed: " #cond " ", ##__VA_ARGS__)
+#define XCHECK(cond, ...)         \
+  XLOG_IF(                        \
+      FATAL,                      \
+      FOLLY_UNLIKELY(!(cond)),    \
+      "Check failed: " #cond " ", \
+      ##__VA_ARGS__)
 
 namespace folly {
 namespace detail {
@@ -582,7 +676,7 @@ std::unique_ptr<std::string> XCheckOpImpl(
     const Arg1& arg1,
     const Arg2& arg2,
     CmpFn&& cmp_fn) {
-  if (LIKELY(cmp_fn(arg1, arg2))) {
+  if (FOLLY_LIKELY(cmp_fn(arg1, arg2))) {
     return nullptr;
   }
   return std::make_unique<std::string>(folly::to<std::string>(
@@ -703,11 +797,11 @@ class XlogLevelInfo {
       XlogFileScopeInfo*) {
     // Do an initial relaxed check.  If this fails we know the category level
     // is initialized and the log admittance check failed.
-    // Use LIKELY() to optimize for the case of disabled debug statements:
+    // Use FOLLY_LIKELY() to optimize for the case of disabled debug statements:
     // we disabled debug statements to be cheap.  If the log message is
     // enabled then this check will still be minimal perf overhead compared to
     // the overall cost of logging it.
-    if (LIKELY(levelToCheck < level_.load(std::memory_order_relaxed))) {
+    if (FOLLY_LIKELY(levelToCheck < level_.load(std::memory_order_relaxed))) {
       return false;
     }
 
@@ -769,7 +863,7 @@ class XlogLevelInfo<false> {
       XlogFileScopeInfo* fileScopeInfo) {
     // As above in the non-specialized XlogFileScopeInfo code, do a simple
     // relaxed check first.
-    if (LIKELY(
+    if (FOLLY_LIKELY(
             levelToCheck <
             fileScopeInfo->level.load(::std::memory_order_relaxed))) {
       return false;
