@@ -862,13 +862,11 @@ inline void swap(
 }
 
 template <typename T>
-struct is_sorted_vector_set : std::false_type {};
-
-template <typename... T>
-struct is_sorted_vector_set<folly::sorted_vector_set<T...>> : std::true_type {};
+FOLLY_INLINE_VARIABLE constexpr bool is_sorted_vector_set_v =
+    detail::is_instantiation_of_v<sorted_vector_set, T>;
 
 template <typename T>
-constexpr bool is_sorted_vector_set_v = is_sorted_vector_set<T>::value;
+struct is_sorted_vector_set : bool_constant<is_sorted_vector_set_v<T>> {};
 
 #if FOLLY_HAS_MEMORY_RESOURCE
 
@@ -1218,6 +1216,24 @@ class sorted_vector_map : detail::growth_policy_wrapper<GrowthPolicy> {
     return try_emplace_impl(k, k, std::forward<Args>(args)...);
   }
 
+  template <typename M>
+  std::pair<iterator, bool> insert_or_assign(const key_type& k, M&& obj) {
+    auto itAndInserted = try_emplace(k, std::forward<M>(obj));
+    if (!itAndInserted.second) {
+      itAndInserted.first->second = std::forward<M>(obj);
+    }
+    return itAndInserted;
+  }
+
+  template <typename M>
+  std::pair<iterator, bool> insert_or_assign(key_type&& k, M&& obj) {
+    auto itAndInserted = try_emplace(std::move(k), std::forward<M>(obj));
+    if (!itAndInserted.second) {
+      itAndInserted.first->second = std::forward<M>(obj);
+    }
+    return itAndInserted;
+  }
+
   size_type erase(const key_type& key) {
     iterator it = find(key);
     if (it == end()) {
@@ -1545,7 +1561,6 @@ class sorted_vector_map : detail::growth_policy_wrapper<GrowthPolicy> {
       its.first = end;
     }
     return its;
-    ;
   }
 
   template <typename... Args>
@@ -1568,13 +1583,11 @@ inline void swap(
 }
 
 template <typename T>
-struct is_sorted_vector_map : std::false_type {};
-
-template <typename... T>
-struct is_sorted_vector_map<folly::sorted_vector_map<T...>> : std::true_type {};
+FOLLY_INLINE_VARIABLE constexpr bool is_sorted_vector_map_v =
+    detail::is_instantiation_of_v<sorted_vector_map, T>;
 
 template <typename T>
-constexpr bool is_sorted_vector_map_v = is_sorted_vector_map<T>::value;
+struct is_sorted_vector_map : bool_constant<is_sorted_vector_map_v<T>> {};
 
 #if FOLLY_HAS_MEMORY_RESOURCE
 
