@@ -16,8 +16,12 @@
 
 #include <folly/synchronization/WaitOptions.h>
 
+#include <folly/Portability.h>
+
 namespace folly {
 
+#if FOLLY_CPLUSPLUS < 201703L
 constexpr std::chrono::nanoseconds WaitOptions::Defaults::spin_max;
+#endif
 
 } // namespace folly
