@@ -28,7 +28,7 @@ using namespace folly;
 using namespace folly::test;
 using namespace testing;
 
-TEST(IndexedMemPool, unique_ptr) {
+TEST(IndexedMemPool, uniquePtr) {
   typedef IndexedMemPool<size_t> Pool;
   Pool pool(100);
 
@@ -50,7 +50,7 @@ TEST(IndexedMemPool, unique_ptr) {
   }
 }
 
-TEST(IndexedMemPool, no_starvation) {
+TEST(IndexedMemPool, noStarvation) {
   const int count = 1000;
   const uint32_t poolSize = 100;
 
@@ -106,7 +106,7 @@ TEST(IndexedMemPool, no_starvation) {
   }
 }
 
-TEST(IndexedMemPool, st_capacity) {
+TEST(IndexedMemPool, stCapacity) {
   // only one local list => capacity is exact
   typedef IndexedMemPool<int, 1, 32> Pool;
   Pool pool(10);
@@ -119,7 +119,7 @@ TEST(IndexedMemPool, st_capacity) {
   EXPECT_EQ(pool.allocIndex(), 0u);
 }
 
-TEST(IndexedMemPool, mt_capacity) {
+TEST(IndexedMemPool, mtCapacity) {
   typedef IndexedMemPool<int, 16, 32> Pool;
   Pool pool(1000);
 
@@ -143,7 +143,7 @@ TEST(IndexedMemPool, mt_capacity) {
   EXPECT_EQ(pool.allocIndex(), 0u);
 }
 
-TEST(IndexedMemPool, locate_elem) {
+TEST(IndexedMemPool, locateElem) {
   IndexedMemPool<int> pool(1000);
 
   for (auto i = 0; i < 1000; ++i) {
@@ -176,7 +176,7 @@ struct NonTrivialStruct {
 
 thread_local size_t NonTrivialStruct::count;
 
-TEST(IndexedMemPool, eager_recycle) {
+TEST(IndexedMemPool, eagerRecycle) {
   typedef IndexedMemPool<NonTrivialStruct> Pool;
   Pool pool(100);
 
@@ -194,7 +194,7 @@ TEST(IndexedMemPool, eager_recycle) {
   }
 }
 
-TEST(IndexedMemPool, late_recycle) {
+TEST(IndexedMemPool, lateRecycle) {
   {
     using Pool = IndexedMemPool<
         NonTrivialStruct,
@@ -219,7 +219,7 @@ TEST(IndexedMemPool, late_recycle) {
   EXPECT_EQ(NonTrivialStruct::count, 0);
 }
 
-TEST(IndexedMemPool, no_data_races) {
+TEST(IndexedMemPool, noDataRaces) {
   const int count = 1000;
   const uint32_t poolSize = 100;
   const int nthreads = 10;
@@ -248,7 +248,7 @@ TEST(IndexedMemPool, no_data_races) {
 std::atomic<int> cnum{0};
 std::atomic<int> dnum{0};
 
-TEST(IndexedMemPool, construction_destruction) {
+TEST(IndexedMemPool, constructionDestruction) {
   struct Foo {
     Foo() { cnum.fetch_add(1); }
     ~Foo() { dnum.fetch_add(1); }
@@ -351,13 +351,13 @@ void testTraits(TraitsTestPool& pool) {
 }
 
 // Test that Traits is used when both local and global lists are empty.
-TEST(IndexedMemPool, use_traits_empty) {
+TEST(IndexedMemPool, useTraitsEmpty) {
   TraitsTestPool pool(10);
   testTraits(pool);
 }
 
 // Test that Traits is used when allocating from a local list.
-TEST(IndexedMemPool, use_traits_local_list) {
+TEST(IndexedMemPool, useTraitsLocalList) {
   TraitsTestPool pool(10);
   MockTraits traits;
   EXPECT_CALL(traits, onAllocate(_, _));
@@ -367,7 +367,7 @@ TEST(IndexedMemPool, use_traits_local_list) {
 }
 
 // Test that Traits is used when allocating from a global list.
-TEST(IndexedMemPool, use_traits_global_list) {
+TEST(IndexedMemPool, useTraitsGlobalList) {
   TraitsTestPool pool(10);
   MockTraits traits;
   EXPECT_CALL(traits, onAllocate(_, _)).Times(2);
