@@ -49,6 +49,9 @@ CPP_assert(ranges::view_<folly::StringPiece>);
 using namespace folly;
 using namespace std;
 
+static_assert(folly::detail::range_is_char_type_v_<char*>, "");
+static_assert(folly::detail::range_is_byte_type_v_<unsigned char*>, "");
+
 static_assert(std::is_literal_type<StringPiece>::value, "");
 
 BOOST_CONCEPT_ASSERT((boost::RandomAccessRangeConcept<StringPiece>));
@@ -531,7 +534,7 @@ TEST(StringPiece, erase) {
   EXPECT_EQ(a, "hello");
 }
 
-TEST(StringPiece, split_step_char_delimiter) {
+TEST(StringPiece, splitStepCharDelimiter) {
   //              0         1         2
   //              012345678901234567890123456
   auto const s = "this is just  a test string";
@@ -589,7 +592,7 @@ TEST(StringPiece, split_step_char_delimiter) {
   EXPECT_EQ("", x);
 }
 
-TEST(StringPiece, split_step_range_delimiter) {
+TEST(StringPiece, splitStepRangeDelimiter) {
   //              0         1         2         3
   //              0123456789012345678901234567890123
   auto const s = "this  is  just    a   test  string";
@@ -654,7 +657,7 @@ TEST(StringPiece, split_step_range_delimiter) {
 
 void split_step_with_process_noop(folly::StringPiece) {}
 
-TEST(StringPiece, split_step_with_process_char_delimiter) {
+TEST(StringPiece, splitStepWithProcessCharDelimiter) {
   //              0         1         2
   //              012345678901234567890123456
   auto const s = "this is just  a test string";
@@ -737,7 +740,7 @@ TEST(StringPiece, split_step_with_process_char_delimiter) {
   EXPECT_NO_THROW(p.split_step(' ', split_step_with_process_noop));
 }
 
-TEST(StringPiece, split_step_with_process_range_delimiter) {
+TEST(StringPiece, splitStepWithProcessRangeDelimiter) {
   //              0         1         2         3
   //              0123456789012345678901234567890123
   auto const s = "this  is  just    a   test  string";
@@ -827,7 +830,7 @@ TEST(StringPiece, split_step_with_process_range_delimiter) {
   EXPECT_NO_THROW(p.split_step(' ', split_step_with_process_noop));
 }
 
-TEST(StringPiece, split_step_with_process_char_delimiter_additional_args) {
+TEST(StringPiece, splitStepWithProcessCharDelimiterAdditionalArgs) {
   //              0         1         2
   //              012345678901234567890123456
   auto const s = "this is just  a test string";
@@ -862,7 +865,7 @@ TEST(StringPiece, split_step_with_process_char_delimiter_additional_args) {
   EXPECT_TRUE(p.empty());
 }
 
-TEST(StringPiece, split_step_with_process_range_delimiter_additional_args) {
+TEST(StringPiece, splitStepWithProcessRangeDelimiterAdditionalArgs) {
   //              0         1         2         3
   //              0123456789012345678901234567890123
   auto const s = "this  is  just    a   test  string";
@@ -907,7 +910,7 @@ TEST(StringPiece, NoInvalidImplicitConversions) {
   EXPECT_TRUE(IsString()(s));
 }
 
-TEST(qfind, UInt32_Ranges) {
+TEST(qfind, UInt32Ranges) {
   vector<uint32_t> a({1, 2, 3, 260, 5});
   vector<uint32_t> b({2, 3, 4});
 
@@ -1257,6 +1260,41 @@ TEST(CRangeFunc, Collection) {
   EXPECT_THAT(numCollRange, testing::ElementsAreArray({17, 1}));
 }
 
+TEST(Range, CompareChar) {
+  EXPECT_EQ(""_sp, ""_sp);
+  EXPECT_LT(""_sp, "world"_sp);
+  EXPECT_GT("world"_sp, ""_sp);
+  EXPECT_EQ("hello"_sp, "hello"_sp);
+  EXPECT_LT("hello"_sp, "world"_sp);
+  EXPECT_LT("hello"_sp, "helloworld"_sp);
+  EXPECT_GT("world"_sp, "hello"_sp);
+  EXPECT_GT("helloworld"_sp, "hello"_sp);
+}
+
+TEST(Range, CompareByte) {
+  auto br = [](auto sp) { return ByteRange(sp); };
+  EXPECT_EQ(br(""_sp), br(""_sp));
+  EXPECT_LT(br(""_sp), br("world"_sp));
+  EXPECT_GT(br("world"_sp), br(""_sp));
+  EXPECT_EQ(br("hello"_sp), br("hello"_sp));
+  EXPECT_LT(br("hello"_sp), br("world"_sp));
+  EXPECT_LT(br("hello"_sp), br("helloworld"_sp));
+  EXPECT_GT(br("world"_sp), br("hello"_sp));
+  EXPECT_GT(br("helloworld"_sp), br("hello"_sp));
+}
+
+TEST(Range, CompareFbck) {
+  auto vr = [](std::vector<int> const& _) { return folly::range(_); };
+  EXPECT_EQ(vr({}), vr({}));
+  EXPECT_LT(vr({}), vr({1}));
+  EXPECT_GT(vr({1}), vr({}));
+  EXPECT_EQ(vr({1}), vr({1}));
+  EXPECT_LT(vr({1}), vr({2}));
+  EXPECT_LT(vr({1}), vr({1, 2}));
+  EXPECT_GT(vr({2}), vr({1}));
+  EXPECT_GT(vr({1, 1}), vr({1}));
+}
+
 std::string get_rand_str(
     size_t size, std::uniform_int_distribution<>& dist, std::mt19937& gen) {
   std::string ret(size, '\0');
